@@ -1,7 +1,6 @@
 import numpy as np
 from collections import namedtuple
-from pycalphad.core.constants import MIN_SITE_FRACTION
-from pycalphad.core.minimizer import find_solution
+from pycalphad.core.minimizer import SystemSpecification
 
 SolverResult = namedtuple('SolverResult', ['converged', 'x', 'chemical_potentials'])
 
@@ -28,12 +27,14 @@ class SolverBase(object):
 
 
 class Solver(SolverBase):
-    def __init__(self, verbose=False, **options):
+    def __init__(self, verbose=False, remove_metastable=True, **options):
         self.verbose = verbose
+        self.remove_metastable = remove_metastable
 
-    def solve(self, composition_sets, conditions):
+
+    def get_system_spec(self, composition_sets, conditions):
         """
-        Minimize the energy under the specified conditions using the given candidate composition sets.
+        Create a SystemSpecification object for the specified conditions.
 
         Parameters
         ----------
@@ -44,7 +45,7 @@ class Solver(SolverBase):
 
         Returns
         -------
-        SolverResult
+        SystemSpecification
 
         """
         compsets = composition_sets
@@ -75,11 +76,56 @@ class Solver(SolverBase):
                 fixed_statevar_indices.append(statevar_idx)
         free_statevar_indices = np.array(sorted(set(range(num_statevars)) - set(fixed_statevar_indices)), dtype=np.int32)
         fixed_statevar_indices = np.array(fixed_statevar_indices, dtype=np.int32)
-        converged, x, chemical_potentials = \
-            find_solution(compsets, num_statevars, num_components, prescribed_system_amount,
-                          chemical_potentials, free_chemical_potential_indices, fixed_chemical_potential_indices,
-                          prescribed_element_indices, prescribed_elemental_amounts,
-                          free_statevar_indices, fixed_statevar_indices)
+        fixed_stable_compset_indices = np.array([i for i, compset in enumerate(compsets) if compset.fixed], dtype=np.int32)
+        spec = SystemSpecification(num_statevars, num_components, prescribed_system_amount,
+                                   chemical_potentials, prescribed_elemental_amounts,
+                                   prescribed_element_indices,
+                                   free_chemical_potential_indices, free_statevar_indices,
+                                   fixed_chemical_potential_indices, fixed_statevar_indices,
+                                   fixed_stable_compset_indices)
+        return spec
+
+    def solve(self, composition_sets, conditions):
+        """
+        Minimize the energy under the specified conditions using the given candidate composition sets.
+
+        Parameters
+        ----------
+        composition_sets : List[pycalphad.core.composition_set.CompositionSet]
+            List of CompositionSet objects in the starting point. Modified in place.
+        conditions : OrderedDict[str, float]
+            Conditions to satisfy.
+
+        Returns
+        -------
+        SolverResult
+
+        """
+        spec = self.get_system_spec(composition_sets, conditions)
+        state = spec.get_new_state(composition_sets)
+        converged = spec.run_loop(state, 1000)
+
+        if self.remove_metastable:
+            phase_idx = 0
+            compsets_to_remove = []
+            for compset in composition_sets:
+                # Mark unstable phases for removal
+                if compset.NP <= 0.0 and not compset.fixed:
+                    compsets_to_remove.append(int(phase_idx))
+                phase_idx += 1
+            # Watch removal order here, as the indices of composition_sets are changing!
+            for idx in reversed(compsets_to_remove):
+                del composition_sets[idx]
+
+        phase_amt = [compset.NP for compset in composition_sets]
+
+        x = composition_sets[0].dof
+        state_variables = composition_sets[0].phase_record.state_variables
+        num_statevars = len(state_variables)
+        for compset in composition_sets[1:]:
+            x = np.r_[x, compset.dof[num_statevars:]]
+        x = np.r_[x, phase_amt]
+        chemical_potentials = np.array(state.chemical_potentials)
 
         if self.verbose:
             print('Chemical Potentials', chemical_potentials)
