@@ -10,6 +10,7 @@ from collections import namedtuple
 import os
 from pycalphad.variables import Species
 from pycalphad.core.cache import fhash
+from pycalphad.core.utils import recursive_tuplify
 
 
 class DatabaseExportError(Exception):
@@ -17,11 +18,6 @@ class DatabaseExportError(Exception):
     pass
 
 
-def _to_tuple(lst):
-    "Convert nested list to nested tuple. Source: Martijn Pieters on StackOverflow"
-    return tuple(_to_tuple(i) if isinstance(i, list) else i for i in lst)
-
-
 class Phase(object): #pylint: disable=R0903
     """
     Phase in the database.
@@ -54,7 +50,7 @@ class Phase(object): #pylint: disable=R0903
         return 'Phase({0!r})'.format(self.__dict__)
     def __hash__(self):
         return hash((self.name, self.constituents, tuple(self.sublattices),
-                     tuple(sorted(_to_tuple(self.model_hints.items())))))
+                     tuple(sorted(recursive_tuplify(self.model_hints.items())))))
 
 DatabaseFormat = namedtuple('DatabaseFormat', ['read', 'write'])
 format_registry = {}
@@ -66,14 +62,14 @@ class Database(object): #pylint: disable=R0902
 
     Attributes
     ----------
-    elements : list
-        List of elements in database.
-    species : list
-        List of species in database.
+    elements : set
+        Set of elements in database.
+    species : set
+        Set of species in database.
     phases : dict
         Phase objects indexed by their system-local name.
     symbols : dict
-        SymPy objects indexed by their name (FUNCTIONs in Thermo-Calc).
+        SymEngine objects indexed by their name (FUNCTIONs in Thermo-Calc).
     references : dict
         Reference objects indexed by their system-local identifier.
 
@@ -144,6 +140,17 @@ class Database(object): #pylint: disable=R0902
             else:
                 setattr(self, key, value)
 
+    def __deepcopy__(self, memo):
+        copy = type(self)()
+        memo[id(self)] = copy
+        for key, value in self.__dict__.items():
+            if key == '_parameters':
+                copy._parameters = TinyDB(storage=MemoryStorage)
+                copy._parameters.insert_multiple(value.all())
+            else:
+                setattr(copy, key, value)
+        return copy
+
     @staticmethod
     def register_format(fmt, read=None, write=None):
         """
@@ -356,16 +363,18 @@ class Database(object): #pylint: disable=R0902
         local_name : string
             System-local name of the phase.
         global_name : object
-            Abstract representation of symbol, e.g., in SymPy format.
+            Abstract representation of symbol, e.g., in SymEngine format.
 
         Examples
         --------
         None yet.
         """
         self._structure_dict[local_name] = global_name
-    def add_parameter(self, param_type, phase_name, #pylint: disable=R0913
-                      constituent_array, param_order,
-                      param, ref=None, diffusing_species=None, force_insert=True):
+
+    def add_parameter(
+        self, param_type, phase_name, constituent_array, param_order, param, ref=None,
+        diffusing_species=None, force_insert=True, **kwargs,
+        ):
         """
         Add a parameter.
 
@@ -380,13 +389,15 @@ class Database(object): #pylint: disable=R0902
         param_order : int
             Polynomial order of the parameter.
         param : object
-            Abstract representation of the parameter, e.g., in SymPy format.
+            Abstract representation of the parameter, e.g., in SymEngine format.
         ref : str, optional
             Reference for the parameter.
         diffusing_species : str, optional
             (If kinetic parameter) Diffusing species for this parameter.
         force_insert : bool, optional
             If True, inserts into the database immediately. False is a delayed insert (for performance).
+        kwargs : Any
+            Additional metadata to insert into the parameter dictionary
 
         Examples
         --------
@@ -402,6 +413,7 @@ class Database(object): #pylint: disable=R0902
             'diffusing_species': Species(diffusing_species),
             'reference': ref
         }
+        new_parameter.update(kwargs)
         if force_insert:
             self._parameters.insert(new_parameter)
         else:
@@ -415,7 +427,7 @@ class Database(object): #pylint: disable=R0902
         ----------
         phase_name : string
             System-local name of the phase.
-        model_hints : list
+        model_hints : dict
             Structured "hints" for a Model trying to read this phase.
             Hints for major constituents and typedefs (Thermo-Calc) go here.
         sublattices : list
@@ -480,4 +492,3 @@ class Database(object): #pylint: disable=R0902
         result = self._parameters.insert_multiple(self._parameter_queue)
         self._parameter_queue = []
         return result
-
