@@ -3,16 +3,20 @@ The test_database module contains tests for the Database object.
 """
 from io import StringIO
 import pytest
+from importlib_resources import files
 import hashlib
 import os
+import pickle
 from copy import deepcopy
+import numpy as np
 from pyparsing import ParseException
-from sympy import Symbol, Piecewise, And
-from pycalphad import Database, Model, variables as v
+from symengine import Symbol, Piecewise, And, S
+from pycalphad import Database, Model, calculate, variables as v
 from pycalphad.variables import Species
-from pycalphad.io.tdb import expand_keyword
+from pycalphad.io.tdb import expand_keyword, reflow_text, TCPrinter
 from pycalphad.io.tdb import _apply_new_symbol_names, DatabaseExportError
-from pycalphad.tests.datasets import ALCRNI_TDB, ALFE_TDB, ALNIPT_TDB, ROSE_TDB, DIFFUSION_TDB
+import pycalphad.tests.databases
+from pycalphad.tests.fixtures import select_database, load_database
 
 
 #
@@ -23,7 +27,7 @@ from pycalphad.tests.datasets import ALCRNI_TDB, ALFE_TDB, ALNIPT_TDB, ROSE_TDB,
 # Underneath it's calling many of the same routines, so we can't guarantee
 # the Database is correct; that's okay, other tests check correctness.
 # We're only checking consistency and exercising error checking here.
-REFERENCE_DBF = Database(ALCRNI_TDB)
+REFERENCE_DBF = Database(str(files(pycalphad.tests.databases).joinpath("alcrni.tdb")))
 REFERENCE_MOD = Model(REFERENCE_DBF, ['CR', 'NI'], 'L12_FCC')
 
 INVALID_TDB_STR="""$ Note: database that invalidates the minimum compatibility subset for TDBs in different softwares
@@ -32,32 +36,43 @@ FUNCTION A_VERY_LONG_FUNCTION_NAME  298.15 -42; 6000 N !
 FUNCTION COMPAT 298.15 +9001; 6000 N !
 """
 
-def test_database_eq():
+@select_database("rose.tdb")
+def test_database_eq(load_database):
     "Database equality comparison."
-    test_dbf = Database(ALCRNI_TDB)
+    test_dbf = Database(str(files(pycalphad.tests.databases).joinpath("alcrni.tdb")))
     assert test_dbf == test_dbf
     assert test_dbf == REFERENCE_DBF
-    assert not (test_dbf == Database(ROSE_TDB))
+    assert not (test_dbf == load_database())
     # literals which don't have __dict__
     assert not (test_dbf == 42)
     assert not (test_dbf == None)
     assert not (42 == test_dbf)
     assert not (None == test_dbf)
 
-def test_database_ne():
+
+@select_database("rose.tdb")
+def test_database_ne(load_database):
     "Database inequality comparison."
-    test_dbf = Database(ALCRNI_TDB)
+    test_dbf = Database(str(files(pycalphad.tests.databases).joinpath("alcrni.tdb")))
     assert not (test_dbf != test_dbf)
     assert not (test_dbf != REFERENCE_DBF)
-    assert test_dbf != Database(ROSE_TDB)
+    assert test_dbf != load_database()
     # literals which don't have __dict__
     assert test_dbf != 42
     assert test_dbf != None
     assert None != test_dbf
     assert 42 != test_dbf
 
+def test_database_pickle():
+    "Database pickle roundtrip."
+    test_dbf = Database(str(files(pycalphad.tests.databases).joinpath("alcrni.tdb")))
+    new_dbf = pickle.loads(pickle.dumps(test_dbf))
+    assert test_dbf == new_dbf
+
+
 def test_database_diffusion():
     "Diffusion database support."
+    DIFFUSION_TDB = open(str(files(pycalphad.tests.databases).joinpath("diffusion.tdb")), "r").read()
     assert Database(DIFFUSION_TDB).phases == \
            Database.from_string(Database(DIFFUSION_TDB).to_string(fmt='tdb'), fmt='tdb').phases
     # Won't work until sympy/sympy#10560 is fixed to prevent precision loss
@@ -65,14 +80,17 @@ def test_database_diffusion():
 
 def test_load_from_string():
     "Test database loading from a string."
-    test_model = Model(Database.from_string(ALCRNI_TDB, fmt='tdb'), ['CR', 'NI'], 'L12_FCC')
+    test_model = Model(Database.from_string(open(files(pycalphad.tests.databases).joinpath("alcrni.tdb"), "r").read()
+                                            , fmt='tdb'), ['CR', 'NI'], 'L12_FCC')
     assert test_model == REFERENCE_MOD
 
-def test_export_import():
+
+@select_database("alfe.tdb")
+def test_export_import(load_database):
     "Equivalence of re-imported database to original."
-    test_dbf = Database(ALNIPT_TDB)
+    test_dbf = Database(str(files(pycalphad.tests.databases).joinpath("alnipt.tdb")))
     assert Database.from_string(test_dbf.to_string(fmt='tdb', if_incompatible='ignore'), fmt='tdb') == test_dbf
-    test_dbf = Database(ALFE_TDB)
+    test_dbf = load_database()
     assert Database.from_string(test_dbf.to_string(fmt='tdb'), fmt='tdb') == test_dbf
 
 def test_incompatible_db_warns_by_default():
@@ -147,27 +165,33 @@ def _testwritetdb():
     yield fname  # run the test
     os.remove(fname)
 
-def test_to_file_defaults_to_raise_if_exists(_testwritetdb):
+
+@select_database("alnipt.tdb")
+def test_to_file_defaults_to_raise_if_exists(load_database, _testwritetdb):
     "Attempting to use Database.to_file should raise by default if it exists"
     fname = _testwritetdb
-    test_dbf = Database(ALNIPT_TDB)
+    test_dbf = load_database()
     test_dbf.to_file(fname)  # establish the initial file
     with pytest.raises(FileExistsError):
         test_dbf.to_file(fname)  # test if_exists behavior
 
-def test_to_file_raises_with_bad_if_exists_argument(_testwritetdb):
+
+@select_database("alnipt.tdb")
+def test_to_file_raises_with_bad_if_exists_argument(load_database, _testwritetdb):
     "Database.to_file should raise if a bad behavior string is passed to if_exists"
     fname = _testwritetdb
-    test_dbf = Database(ALNIPT_TDB)
+    test_dbf = load_database()
     test_dbf.to_file(fname)  # establish the initial file
     with pytest.raises(FileExistsError):
         test_dbf.to_file(fname, if_exists='TEST_BAD_ARGUMENT')  # test if_exists behavior
 
-def test_to_file_overwrites_with_if_exists_argument(_testwritetdb):
+
+@select_database("alnipt.tdb")
+def test_to_file_overwrites_with_if_exists_argument(load_database, _testwritetdb):
     "Database.to_file should overwrite if 'overwrite' is passed to if_exists"
     import time
     fname = _testwritetdb
-    test_dbf = Database(ALNIPT_TDB)
+    test_dbf = load_database()
     test_dbf.to_file(fname)  # establish the initial file
     inital_modification_time = os.path.getmtime(fname)
     time.sleep(1)  # this test can fail intermittently without waiting.
@@ -175,15 +199,16 @@ def test_to_file_overwrites_with_if_exists_argument(_testwritetdb):
     overwrite_modification_time = os.path.getmtime(fname)
     assert overwrite_modification_time > inital_modification_time
 
+
 def test_unspecified_format_from_string():
     "from_string: Unspecified string format raises ValueError."
     with pytest.raises(ValueError):
-        Database.from_string(ALCRNI_TDB)
+        Database.from_string(str(files(pycalphad.tests.databases).joinpath("alcrni.tdb")))
 
 def test_unknown_format_from_string():
     "from_string: Unknown import string format raises NotImplementedError."
     with pytest.raises(NotImplementedError):
-        Database.from_string(ALCRNI_TDB, fmt='_fail_')
+        Database.from_string(str(files(pycalphad.tests.databases).joinpath("alcrni.tdb")), fmt='_fail_')
 
 def test_unknown_format_to_string():
     "to_string: Unknown export file format raises NotImplementedError."
@@ -192,18 +217,18 @@ def test_unknown_format_to_string():
 
 def test_load_from_stringio():
     "Test database loading from a file-like object."
-    test_tdb = Database(StringIO(ALCRNI_TDB))
+    test_tdb = Database(StringIO(open(str(files(pycalphad.tests.databases).joinpath("alcrni.tdb")), "r").read()))
     assert test_tdb == REFERENCE_DBF
 
 def test_load_from_stringio_from_file():
     "Test database loading from a file-like object with the from_file method."
-    test_tdb = Database.from_file(StringIO(ALCRNI_TDB), fmt='tdb')
+    test_tdb = Database.from_file(StringIO(open(str(files(pycalphad.tests.databases).joinpath("alcrni.tdb")), "r").read()), fmt='tdb')
     assert test_tdb == REFERENCE_DBF
 
 def test_unspecified_format_from_file():
     "from_file: Unspecified format for file descriptor raises ValueError."
     with pytest.raises(ValueError):
-        Database.from_file(StringIO(ALCRNI_TDB))
+        Database.from_file(StringIO(str(files(pycalphad.tests.databases).joinpath("alcrni.tdb"))))
 
 def test_unspecified_format_to_file():
     "to_file: Unspecified format for file descriptor raises ValueError."
@@ -213,7 +238,7 @@ def test_unspecified_format_to_file():
 def test_unknown_format_from_file():
     "from_string: Unknown import file format raises NotImplementedError."
     with pytest.raises(NotImplementedError):
-        Database.from_string(ALCRNI_TDB, fmt='_fail_')
+        Database.from_string(str(files(pycalphad.tests.databases).joinpath("alcrni.tdb")), fmt='_fail_')
 
 def test_unknown_format_to_file():
     "to_file: Unknown export file format raises NotImplementedError."
@@ -542,19 +567,19 @@ def test_database_parameter_with_species_that_is_not_a_stoichiometric_formula():
     import tinydb
     silica = dbf._parameters.search(tinydb.where('constituent_array') == ((species_dict['SILICA'],),))
     assert len(silica) == 1
-    assert silica[0]['parameter'].args[0][0] == 10
+    assert silica[0]['parameter'].args[0] == float(10)
 
     nasb_6oh = dbf._parameters.search(tinydb.where('constituent_array') == ((species_dict['NASB_6OH'],),))
     assert len(nasb_6oh) == 1
-    assert nasb_6oh[0]['parameter'].args[0][0] == 100
+    assert nasb_6oh[0]['parameter'].args[0] == float(100)
 
     alcl2oh_3water = dbf._parameters.search(tinydb.where('constituent_array') == ((species_dict['ALCL2OH.3WATER'],),))
     assert len(alcl2oh_3water) == 1
-    assert alcl2oh_3water[0]['parameter'].args[0][0] == 1000
+    assert alcl2oh_3water[0]['parameter'].args[0] == float(1000)
 
     sbminus3 = dbf._parameters.search(tinydb.where('constituent_array') == ((species_dict['SB-3'],),))
     assert len(sbminus3) == 1
-    assert sbminus3[0]['parameter'].args[0][0] == 10000
+    assert sbminus3[0]['parameter'].args[0] == float(10000)
 
 
 def test_database_sympy_namespace_clash():
@@ -679,9 +704,160 @@ def test_tdb_parser_raises_unterminated_parameters():
     with pytest.raises(ParseException):
         Database(UNTERMINATED_PARAM_STR)
 
-def test_load_database_when_given_in_lowercase():
+
+@select_database("alfe.tdb")
+def test_load_database_when_given_in_lowercase(load_database):
     "Test loading a database coerced to lowercase loads correctly."
+    ALFE_TDB = load_database().to_string(fmt='tdb')
     dbf = Database.from_string(ALFE_TDB, fmt='tdb')
     dbf_lower = Database.from_string(ALFE_TDB.lower(), fmt='tdb')
 
-    assert dbf == dbf_lower
\ No newline at end of file
+    assert dbf == dbf_lower
+
+
+def test_reflow_text_raises_on_unbreakable_lines():
+    """Long lines without linebreak characters should raise an error."""
+    very_long_line = "NOBREAKCHARACTERS" * 50
+    with pytest.raises(ValueError):
+        reflow_text(very_long_line, 80)
+
+
+# TODO: when the new database-as-files test fixture is merged, replace with unary 50 proper.
+def test_long_constituent_line_writes_correctly():
+    TDB = """
+    $ From SGTE Unary 50
+    ELEMENT /-   ELECTRON_GAS              0.0000E+00  0.0000E+00  0.0000E+00 !
+    ELEMENT VA   VACUUM                    0.0000E+00  0.0000E+00  0.0000E+00 !
+    ELEMENT AG   FCC_A1                    1.0787E+02  5.7446E+03  4.2551E+01 !
+    ELEMENT AL   FCC_A1                    2.6982E+01  4.5773E+03  2.8322E+01 !
+    ELEMENT AM   DHCP                      2.4306E+02  0.0000E+00  0.0000E+00 !
+    ELEMENT AS   RHOMBOHEDRAL_A7           7.4922E+01  0.0000E+00  0.0000E+00 !
+    ELEMENT AU   FCC_A1                    1.9697E+02  6.0166E+03  4.7488E+01 !
+    ELEMENT B    BETA_RHOMBO_B             1.0811E+01  1.2220E+03  5.9000E+00 !
+    ELEMENT BA   BCC_A2                    1.3733E+02  0.0000E+00  0.0000E+00 !
+    ELEMENT BE   HCP_A3                    9.0122E+00  0.0000E+00  0.0000E+00 !
+    ELEMENT BI   RHOMBOHEDRAL_A7           2.0898E+02  6.4266E+03  5.6735E+01 !
+    ELEMENT C    GRAPHITE                  1.2011E+01  1.0540E+03  5.7400E+00 !
+    ELEMENT CA   FCC_A1                    4.0078E+01  6.1965E+03  4.1589E+01 !
+    ELEMENT CD   HCP_A3                    1.1241E+02  6.2509E+03  5.1798E+01 !
+    ELEMENT CE   FCC_A1                    1.4011E+02  0.0000E+00  0.0000E+00 !
+    ELEMENT CO   HCP_A3                    5.8933E+01  0.0000E+00  0.0000E+00 !
+    ELEMENT CR   BCC_A2                    5.1996E+01  4.0500E+03  2.3560E+01 !
+    ELEMENT CS   BCC_A2                    1.3291E+02  7.7153E+03  8.5149E+01 !
+    ELEMENT CU   FCC_A1                    6.3546E+01  5.0041E+03  3.3150E+01 !
+    ELEMENT DY   HCP_A3                    1.6250E+02  0.0000E+00  0.0000E+00 !
+    ELEMENT ER   HCP_A3                    1.6726E+02  7.3923E+03  7.3178E+01 !
+    ELEMENT EU   BCC_A2                    1.5197E+02  0.0000E+00  8.0793E+01 !
+    ELEMENT FE   BCC_A2                    5.5847E+01  4.4890E+03  2.7280E+01 !
+    ELEMENT GA   ORTHORHOMBIC_GA           6.9723E+01  5.5731E+03  4.0828E+01 !
+    ELEMENT GD   HCP_A3                    1.5725E+02  0.0000E+00  0.0000E+00 !
+    ELEMENT GE   DIAMOND_A4                7.2610E+01  4.6275E+03  3.1087E+01 !
+    ELEMENT HF   HCP_A3                    1.7849E+02  0.0000E+00  0.0000E+00 !
+    ELEMENT HG   LIQUID                    2.0059E+02  0.0000E+00  0.0000E+00 !
+    ELEMENT HO   HCP_A3                    1.6493E+02  0.0000E+00  0.0000E+00 !
+    ELEMENT IN   TETRAGONAL_A6             1.1482E+02  6.6100E+03  5.7650E+01 !
+    ELEMENT IR   FCC_A1                    1.9222E+02  5.2677E+03  3.5505E+01 !
+    ELEMENT K    BCC_A2                    3.9098E+01  7.0835E+03  6.4672E+01 !
+    ELEMENT LA   DHCP                      1.3891E+02  0.0000E+00  0.0000E+00 !
+    ELEMENT LI   BCC_A2                    6.9410E+00  4.6233E+03  2.9095E+01 !
+    ELEMENT LU   HCP_A3                    1.7497E+02  0.0000E+00  0.0000E+00 !
+    ELEMENT MG   HCP_A3                    2.4305E+01  4.9980E+03  3.2671E+01 !
+    ELEMENT MN   CBCC_A12                  5.4938E+01  4.9960E+03  3.2008E+01 !
+    ELEMENT MO   BCC_A2                    9.5940E+01  4.5890E+03  2.8560E+01 !
+    ELEMENT N    1/2_MOLE_N2(G)            1.4007E+01  4.3350E+03  9.5751E+01 !
+    ELEMENT NA   BCC_A2                    2.2990E+01  6.4475E+03  5.1447E+01 !
+    ELEMENT NB   BCC_A2                    9.2906E+01  5.2200E+03  3.6270E+01 !
+    ELEMENT ND   DHCP                      1.4424E+02  0.0000E+00  0.0000E+00 !
+    ELEMENT NI   FCC_A1                    5.8690E+01  4.7870E+03  2.9796E+01 !
+    ELEMENT NP   ORTHORHOMBIC_AC           2.3705E+02  0.0000E+00  0.0000E+00 !
+    ELEMENT O    1/2_MOLE_O2(G)            1.5999E+01  4.3410E+03  1.0252E+02 !
+    ELEMENT OS   HCP_A3                    1.9020E+02  0.0000E+00  3.2635E+01 !
+    ELEMENT P    WHITE_P                   3.0974E+01  0.0000E+00  0.0000E+00 !
+    ELEMENT PA   BCT_AA                    2.3104E+02  0.0000E+00  0.0000E+00 !
+    ELEMENT PB   FCC_A1                    2.0720E+02  6.8785E+03  6.4785E+01 !
+    ELEMENT PD   FCC_A1                    1.0642E+02  5.4685E+03  3.7823E+01 !
+    ELEMENT PR   DHCP                      1.4091E+02  0.0000E+00  0.0000E+00 !
+    ELEMENT PT   FCC_A1                    1.9508E+02  5.7237E+03  4.1631E+01 !
+    ELEMENT PU   ALPHA_PU                  2.4406E+02  0.0000E+00  0.0000E+00 !
+    ELEMENT RB   BCC_A2                    8.5468E+01  7.4894E+03  7.6776E+01 !
+    ELEMENT RE   HCP_A3                    1.8621E+02  5.3555E+03  3.6526E+01 !
+    ELEMENT RH   FCC_A1                    1.0291E+02  4.9204E+03  3.1505E+01 !
+    ELEMENT RU   HCP_A3                    1.0107E+02  4.6024E+03  2.8535E+01 !
+    ELEMENT S    ORTHORHOMBIC_S            3.2066E+01  0.0000E+00  0.0000E+00 !
+    ELEMENT SB   RHOMBOHEDRAL_A7           1.2175E+02  5.8702E+03  4.5522E+01 !
+    ELEMENT SC   HCP_A3                    4.4956E+01  0.0000E+00  0.0000E+00 !
+    ELEMENT SE   HEXAGONAL_A8              7.8960E+01  5.5145E+03  4.1966E+01 !
+    ELEMENT SI   DIAMOND_A4                2.8085E+01  3.2175E+03  1.8820E+01 !
+    ELEMENT SM   RHOMBOHEDRAL_SM           1.5036E+02  0.0000E+00  0.0000E+00 !
+    ELEMENT SN   BCT_A5                    1.1871E+02  6.3220E+03  5.1195E+01 !
+    ELEMENT SR   FCC_A1                    8.7620E+01  0.0000E+00  0.0000E+00 !
+    ELEMENT TA   BCC_A2                    1.8095E+02  5.6819E+03  4.1472E+01 !
+    ELEMENT TB   HCP_A3                    1.5893E+02  0.0000E+00  0.0000E+00 !
+    ELEMENT TC   HCP_A3                    9.7907E+01  0.0000E+00  0.0000E+00 !
+    ELEMENT TE   HEXAGONAL_A8              1.2760E+02  6.1212E+03  4.9497E+01 !
+    ELEMENT TH   FCC_A1                    2.3204E+02  0.0000E+00  0.0000E+00 !
+    ELEMENT TI   HCP_A3                    4.7880E+01  4.8100E+03  3.0648E+01 !
+    ELEMENT TL   HCP_A3                    2.0438E+02  6.8283E+03  6.4183E+01 !
+    ELEMENT TM   HCP_A3                    1.6893E+02  7.3973E+03  7.4015E+01 !
+    ELEMENT U    ORTHORHOMBIC_A20          2.3803E+02  0.0000E+00  0.0000E+00 !
+    ELEMENT V    BCC_A2                    5.0941E+01  4.5070E+03  3.0890E+01 !
+    ELEMENT W    BCC_A2                    1.8385E+02  4.9700E+03  3.2620E+01 !
+    ELEMENT Y    HCP_A3                    8.8906E+01  0.0000E+00  0.0000E+00 !
+    ELEMENT YB   FCC_A1                    1.7304E+02  0.0000E+00  0.0000E+00 !
+    ELEMENT ZN   HCP_A3                    6.5390E+01  5.6568E+03  4.1631E+01 !
+    ELEMENT ZR   HCP_A3                    9.1224E+01  5.5663E+03  3.9181E+01 !
+
+    PHASE LIQUID % 1 1 !
+    CONSTITUENT LIQUID : AG,AL,AM,AS,AU,B,BA,BE,BI,C,CA,CD,CE,CO,CR,CS,CU,DY,ER,
+    EU,FE,GA,GD,GE,HF,HG,HO,IN,IR,K,LA,LI,LU,MG,MN,MO,N,NA,NB,ND,NI,NP,O,OS,P,PA,
+    PB,PD,PR,PT,PU,RB,RE,RH,RU,S,SB,SC,SE,SI,SM,SN,SR,TA,TB,TC,TE,TH,TI,TL,TM,U,V,
+    W,Y,YB,ZN,ZR : !
+    """
+    dbf = Database(TDB)
+    assert len(dbf.elements) == 80
+    assert len(dbf.phases['LIQUID'].constituents[0]) == 78  # No VA or /-
+    reloaded_dbf = Database(dbf.to_string(fmt='tdb'))
+    dbf == reloaded_dbf
+    assert len(dbf.elements) == len(reloaded_dbf.elements)
+    assert len(dbf.phases['LIQUID'].constituents[0]) == len(reloaded_dbf.phases['LIQUID'].constituents[0])
+
+def test_database_passes_with_diffusion_commands():
+    "DIFFUSION and ZEROVOLUME_SPECIES commands do not raise errors while parsing (doesn't test implementation)."
+    tdb_string = """
+     ELEMENT /-   ELECTRON_GAS              0.0000E+00  0.0000E+00  0.0000E+00 !
+     ELEMENT VA   VACUUM                    0.0000E+00  0.0000E+00  0.0000E+00 !
+
+     ZEROVOLUME_SPECIES VA !
+     DIFFUSION MAGNETIC BCC_A2 ALPHA=0.3 !
+    """
+
+    dbf = Database.from_string(tdb_string, fmt='tdb')
+
+def test_tc_printer_no_division_symbols():
+    "TCPrinter does not produce division symbols in string output of symbolic expressions."
+    test_expr = Piecewise((S('VV0000/T + T*VV0004 + T**2*VV0001 + T**3*VV0002 + T*LOG(T)*VV0003'), v.T>0))
+    result = TCPrinter().doprint(test_expr)
+    assert '/' not in result
+
+def test_tc_printer_exp():
+    "TCPrinter prints the exponential function when the argument is not an integer."
+    test_expr = S('exp(-300T**(-1))')
+    result = TCPrinter()._stringify_expr(test_expr)
+    assert result == 'exp(-300 * T**(-1))'
+
+
+@select_database("Al-Fe_sundman2009.tdb")
+def test_database_symmetry_options_are_generated(load_database):
+    """Loading a database with option B and F generates new ordering parameters."""
+    # Tests for the correctness of which configurations are generated is done
+    # by testing generate_symmetric_group
+    # This test aims to see that the parameters are loaded (resulting in a
+    # particular number of parameters) and that removing those parameters is
+    # possible.
+    dbf = load_database()
+    assert len(dbf._parameters) == 375  # number after adding automatically
+
+    # The parameters should be filtered out when when writing such that a
+    # read/write is a no-op
+    read_dbf = Database.from_string(dbf.to_string(fmt="tdb"), fmt="tdb")
+    assert len(read_dbf._parameters) == 375
\ No newline at end of file
